@@ -1,5 +1,5 @@
 from necroassembler import Assembler, opcode
-from necroassembler.utils import pack_bits_le16u, pack_le16u, in_bit_range_signed, pack_bit
+from necroassembler.utils import pack_bits_le16u, pack_le16u, pack_bit
 from necroassembler.exceptions import (
     AssemblerException, InvalidRegister, UnknownRegister, InvalideImmediateValue, NotInBitRange)
 
@@ -59,37 +59,31 @@ class AssemblerThumb(Assembler):
     special_prefixes = ('#',)
 
     def _offset(self, arg, bits, alignment):
-        return self.parse_integer_or_label(arg,
-                                           bits=bits,
-                                           relative=True,
+        return self.parse_integer_or_label(label=arg,
                                            size=2,
-                                           offset=0,
+                                           bits_size=(
+                                               bits[0] - bits[1]) + 1 + (alignment//2),
+                                           bits=bits,
                                            filter=lambda x: x >> (
                                                alignment//2),
                                            alignment=alignment,
-                                           start=self.current_org + self.org_counter + 4)
+                                           relative=self.pc + 4)
 
     def _imm(self, arg):
-        value = self.parse_integer(arg[1:])
+        value = self.parse_integer(arg[1:], 8, False)
         if value is None:
             raise InvalideImmediateValue()
         return value
 
     def _word8(self, arg):
-        value = self.parse_integer(arg[1:])
-        if value is None:
-            pc = self.current_org + self.org_counter + 4
-            self.add_label_translation(label=arg[1:],
-                                       relative=True,
-                                       bits=(7, 0),
-                                       size=2,
-                                       offset=0,
-                                       alignment=4,
-                                       filter=lambda x: x >> 2,
-                                       # bit 1 of pc must be turned off
-                                       start=pc & ~(0b10))
-            value = 0
-        return value
+        return self.parse_integer_or_label(label=arg[1:],
+                                           size=2,
+                                           bits_size=8,
+                                           bits=(7, 0),
+                                           alignment=4,
+                                           filter=lambda x: x >> 2,
+                                           # bit 1 of pc must be turned off
+                                           relative=(self.pc + 4) & ~(0b10))
 
     def _conditional_branch(self, instr, cond):
         if instr.match(LABEL):
@@ -376,36 +370,30 @@ class AssemblerThumb(Assembler):
 
         offset = instr.tokens[1]
 
-        address = self.parse_integer(offset)
+        address = self.parse_integer(offset, 23, signed=True)
         if address:
-            if not in_bit_range_signed(address, 23):
-                raise NotInBitRange(address, 23)
             address >>= 1
             address0 = address >> 11
             address1 = address & 0x7ff
             return self._build_opcode(0b1111000000000000, ((10, 0), address0)) + self._build_opcode(0b1111100000000000, ((10, 0), address1))
 
         self.add_label_translation(label=offset,
-                                   bits=(10, 0),
-                                   relative=True,
                                    size=2,
-                                   offset=0,
+                                   bits_size=23,
+                                   bits=(10, 0),
                                    alignment=2,
-                                   filter=lambda x: x >> 1,
-                                   post_filter=lambda x: x >> 11,
-                                   bits_check=23,
-                                   start=self.current_org + self.org_counter + 4)
+                                   relative=self.pc + 4,
+                                   # get high 11 bits (after >> 1)
+                                   filter=lambda x: x >> 12)
         self.add_label_translation(label=offset,
-                                   bits=(10, 0),
-                                   relative=True,
-                                   size=2,
                                    offset=2,
+                                   size=2,
+                                   bits_size=23,
+                                   bits=(10, 0),
                                    alignment=2,
-                                   filter=lambda x: x >> 1,
-                                   # get first 11 bits
-                                   post_filter=lambda x: x & 0x7FF,
-                                   bits_check=23,
-                                   start=self.current_org + self.org_counter + 4)
+                                   relative=self.pc + 4,
+                                   # get low 11 bits (after >> 1)
+                                   filter=lambda x: ((x >> 1) & 0x7FF))
 
         return pack_le16u(0b1111000000000000, 0b1111100000000000)
 
@@ -413,7 +401,7 @@ class AssemblerThumb(Assembler):
     def _b(self, instr):
         if instr.match(LABEL):
             offset = self._offset(instr.tokens[1], (10, 0), 2)
-            return self._build_opcode(0b1110000000000000, ((10, 0), offset >> 1, True))
+            return self._build_opcode(0b1110000000000000, ((10, 0), offset >> 1))
 
     @opcode('BEQ')
     def _beq(self, instr):
